@@ -16,9 +16,13 @@
 
 """L7rule action implementation"""
 
+import functools
+
 from cliff import lister
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
+from oslo_utils import uuidutils
 
 from octaviaclient.osc.v2 import constants as const
 from octaviaclient.osc.v2 import utils as v2_utils
@@ -85,6 +89,11 @@ class CreateL7Rule(command.ShowOne):
             default=None,
             help="Disable l7rule."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
 
         return parser
 
@@ -101,6 +110,22 @@ class CreateL7Rule(command.ShowOne):
             json=body
         )
 
+        if parsed_args.wait:
+            l7policy = self.app.client_manager.load_balancer.l7policy_show(
+                l7policy_id)
+            listener = self.app.client_manager.load_balancer.listener_show(
+                l7policy['listener_id'])
+            v2_utils.wait_for_active(
+                status_f=(self.app.client_manager.load_balancer.
+                          load_balancer_show),
+                res_id=listener['loadbalancers'][0]['id']
+            )
+            data = {
+                'rule': (
+                    self.app.client_manager.load_balancer.l7rule_show(
+                        l7policy_id, data['rule']['id']))
+            }
+
         return (rows, (utils.get_dict_properties(
             data['rule'], rows, formatters={})))
 
@@ -121,6 +146,11 @@ class DeleteL7Rule(command.Command):
             metavar="<rule_id>",
             help="l7rule to delete."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
 
         return parser
 
@@ -132,6 +162,16 @@ class DeleteL7Rule(command.Command):
             l7policy_id=attrs['l7policy_id']
         )
 
+        if parsed_args.wait:
+            l7rule_show = functools.partial(
+                self.app.client_manager.load_balancer.l7rule_show,
+                attrs['l7policy_id']
+            )
+            v2_utils.wait_for_delete(
+                status_f=l7rule_show,
+                res_id=attrs['l7rule_id']
+            )
+
 
 class ListL7Rule(lister.Lister):
     """List l7rules for l7policy"""
@@ -181,12 +221,22 @@ class ShowL7Rule(command.ShowOne):
 
     def take_action(self, parsed_args):
         rows = const.L7RULE_ROWS
-
-        attrs = v2_utils.get_l7rule_attrs(self.app.client_manager, parsed_args)
-        data = self.app.client_manager.load_balancer.l7rule_show(
-            l7rule_id=attrs['l7rule_id'],
-            l7policy_id=attrs['l7policy_id']
-        )
+        data = None
+        if (uuidutils.is_uuid_like(parsed_args.l7policy) and
+                uuidutils.is_uuid_like(parsed_args.l7rule)):
+            try:
+                data = self.app.client_manager.load_balancer.l7rule_show(
+                    l7rule_id=parsed_args.l7rule,
+                    l7policy_id=parsed_args.l7policy)
+            except exceptions.NotFound:
+                pass
+        if data is None:
+            attrs = v2_utils.get_l7rule_attrs(self.app.client_manager,
+                                              parsed_args)
+            data = self.app.client_manager.load_balancer.l7rule_show(
+                l7rule_id=attrs['l7rule_id'],
+                l7policy_id=attrs['l7policy_id']
+            )
 
         return (rows, (utils.get_dict_properties(
             data, rows, formatters={})))
@@ -251,6 +301,11 @@ class SetL7Rule(command.Command):
             default=None,
             help="Disable l7rule."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
 
         return parser
 
@@ -269,6 +324,16 @@ class SetL7Rule(command.Command):
             json=body
         )
 
+        if parsed_args.wait:
+            l7rule_show = functools.partial(
+                self.app.client_manager.load_balancer.l7rule_show,
+                l7policy_id
+            )
+            v2_utils.wait_for_active(
+                status_f=l7rule_show,
+                res_id=l7rule_id
+            )
+
 
 class UnsetL7Rule(command.Command):
     """Clear l7rule settings"""
@@ -296,6 +361,11 @@ class UnsetL7Rule(command.Command):
             action='store_true',
             help="Clear the l7rule key."
         )
+        parser.add_argument(
+            '--wait',
+            action='store_true',
+            help='Wait for action to complete',
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -311,3 +381,13 @@ class UnsetL7Rule(command.Command):
 
         self.app.client_manager.load_balancer.l7rule_set(
             l7policy_id=policy_id, l7rule_id=parsed_args.l7rule_id, json=body)
+
+        if parsed_args.wait:
+            l7rule_show = functools.partial(
+                self.app.client_manager.load_balancer.l7rule_show,
+                policy_id
+            )
+            v2_utils.wait_for_active(
+                status_f=l7rule_show,
+                res_id=parsed_args.l7rule_id
+            )
