@@ -12,9 +12,13 @@
 #   under the License.
 #
 
-from osc_lib import exceptions
-
+import munch
 from openstackclient.identity import common as identity_common
+from osc_lib import exceptions as osc_exc
+from osc_lib import utils
+
+from octaviaclient.api import exceptions
+from octaviaclient.osc.v2 import constants
 
 
 def _map_attrs(args, source_attr_map):
@@ -76,6 +80,11 @@ def get_resource_id(resource, resource_name, name):
             if name.lower() in ('none', 'null', 'void'):
                 return None
 
+        primary_key = 'id'
+        # Availability-zones don't have an id value
+        if resource_name == 'availability_zones':
+            primary_key = 'name'
+
         # Projects can be non-uuid so we need to account for this
         if resource_name == 'project':
             if name != 'non-uuid':
@@ -95,7 +104,7 @@ def get_resource_id(resource, resource_name, name):
                 msg = ("{0} {1} found with name or ID of {2}. Please try "
                        "again with UUID".format(len(names), resource_name,
                                                 name))
-                raise exceptions.CommandError(msg)
+                raise osc_exc.CommandError(msg)
             else:
                 return names[0].get('id')
         elif resource_name == 'l7rules':
@@ -110,12 +119,13 @@ def get_resource_id(resource, resource_name, name):
                 msg = ("{0} {1} found with name or ID of {2}. Please try "
                        "again with UUID".format(len(names), resource_name,
                                                 name))
-                raise exceptions.CommandError(msg)
+                raise osc_exc.CommandError(msg)
             else:
-                return names[0].get('id')
+                return names[0].get(primary_key)
+
     except IndexError:
         msg = "Unable to locate {0} in {1}".format(name, resource_name)
-        raise exceptions.CommandError(msg)
+        raise osc_exc.CommandError(msg)
 
 
 def get_loadbalancer_attrs(client_manager, parsed_args):
@@ -167,6 +177,8 @@ def get_loadbalancer_attrs(client_manager, parsed_args):
             'flavors',
             client_manager.load_balancer.flavor_list
         ),
+        'availability_zone': ('availability_zone', str),
+
     }
 
     _attrs = vars(parsed_args)
@@ -467,6 +479,8 @@ def get_provider_attrs(parsed_args):
     attr_map = {
         'provider': ('provider_name', str),
         'description': ('description', str),
+        'flavor': ('flavor', bool),
+        'availability_zone': ('availability_zone', bool),
     }
 
     return _map_attrs(vars(parsed_args), attr_map)
@@ -514,6 +528,48 @@ def get_flavorprofile_attrs(client_manager, parsed_args):
     return attrs
 
 
+def get_availabilityzone_attrs(client_manager, parsed_args):
+    attr_map = {
+        'name': ('name', str),
+        'availabilityzone': (
+            'availabilityzone_name',
+            'availability_zones',
+            client_manager.load_balancer.availabilityzone_list,
+        ),
+        'availabilityzoneprofile': (
+            'availability_zone_profile_id',
+            'availability_zone_profiles',
+            client_manager.load_balancer.availabilityzoneprofile_list,
+        ),
+        'enable': ('enabled', lambda x: True),
+        'disable': ('enabled', lambda x: False),
+        'description': ('description', str),
+    }
+
+    _attrs = vars(parsed_args)
+    attrs = _map_attrs(_attrs, attr_map)
+
+    return attrs
+
+
+def get_availabilityzoneprofile_attrs(client_manager, parsed_args):
+    attr_map = {
+        'name': ('name', str),
+        'availabilityzoneprofile': (
+            'availability_zone_profile_id',
+            'availability_zone_profiles',
+            client_manager.load_balancer.availabilityzoneprofile_list,
+        ),
+        'provider': ('provider_name', str),
+        'availability_zone_data': ('availability_zone_data', str),
+    }
+
+    _attrs = vars(parsed_args)
+    attrs = _map_attrs(_attrs, attr_map)
+
+    return attrs
+
+
 def format_list(data):
     return '\n'.join(i['id'] for i in data)
 
@@ -547,4 +603,39 @@ def _format_str_if_need_treat_unset(data):
 
 def get_unsets(parsed_args):
     return {arg: None for arg, value in vars(parsed_args).items() if
-            value is True}
+            value is True and arg != 'wait'}
+
+
+def wait_for_active(status_f, res_id):
+    success = utils.wait_for_status(
+        status_f=lambda x: munch.Munch(status_f(x)),
+        res_id=res_id,
+        status_field=constants.PROVISIONING_STATUS,
+        sleep_time=3
+    )
+    if not success:
+        raise exceptions.OctaviaClientException(
+            code="n/a",
+            message="The resource did not successfully reach ACTIVE status.")
+
+
+def wait_for_delete(status_f, res_id):
+    class Getter(object):
+        @staticmethod
+        def get(id):
+            return munch.Munch(status_f(id))
+
+    try:
+        success = utils.wait_for_delete(
+            manager=Getter,
+            res_id=res_id,
+            status_field=constants.PROVISIONING_STATUS,
+            sleep_time=3
+        )
+        if not success:
+            raise exceptions.OctaviaClientException(
+                code="n/a",
+                message="The resource could not be successfully deleted.")
+    except exceptions.OctaviaClientException as e:
+        if e.code != 404:
+            raise
